/* hyp_1.c - hyperbolic, explicit, one-step */

/*
   Conservative one-step explicit scheme for scalar hyperbolic equation:

   du/dt + df(u)/dx = 0,

	  1) f(x) = u*u/2:  conservative form of nonlinear Burger's equation,
	  2) f(x) = C*u:    linear vave equation.

   Spatial reconstruction:

	  1) piecewise-constant,
	  2) monotone piecewise-parabolic.

   Written by  Andrei A. Chernousov
   E-mail: <andrei99@iname.com>
   Web: http://www.geocities.com/andrei_chernousov
   
*/

#include <stdio.h>
#include <stdlib.h>


/*-- monotone piecewise-parabolic reconstruction --*/
/* constant coefficients, recommended values are 1/3, 4, 1/3, 1/6 */
#define FI 0.333333333333333
#define BB 4.000000000000000
#define C1 0.333333333333333
#define C2 0.166666666666667
/* limiting function - the best: only necessary multiplication *BB */
double ay;
/*inline*/ double minmod( double x, double y ) {  /* Chakravarthy, Osher */
  if( x*y<=0 ) return 0;
  ay = ( (y*=BB) > 0 ) ? y: -y;
  if( x > 0 ) return (x<ay) ? x : ay;
  return ( (-x<ay) ? x: -ay );
} /* end minmod() */

enum {LOW = 1}; /* order = LOW in case of donor cell method */

/*-- flux function --*/
const double C = 0.5;
enum {LINEAR, NONLINEAR};
int funct;

double Fun(double u)  /* f(u) */
{
    if (funct)
	return C * u * u;
    else 
	return C * u;
} /* end Fun */


/*-- main function --*/

int main( )
{
  double
    *u, *F,
    u_min, u_max,
    deltaX, deltaT, deltaT_X,
    m, w, M, W,
    du_dx, du_dx2,
    u0, x;

  int
    i, j,
    LEN, LENN,
    order,
    step, numstep;


  FILE *pF;
  char str[80];

    
  i = 0;
  do {
    if (fgets(str, 80, stdin) != NULL) {
      switch (i) {
      case 0:  LEN    = atoi(str); break;
      case 1:  deltaX = atof(str); break;
      case 2:  deltaT = atof(str); break;
      case 3:  numstep= atoi(str); break;
      case 4:  funct  = atoi(str); break;
      case 5:  order  = atoi(str); break;
      default: break;
      } /* end switch() */
      i++;
    } /* end if() */
    else {
      fprintf(stderr, "ppm: error reading input\n");
      exit(-1);
    }	
  } while (i < 6);

  /**/
  deltaT_X = deltaT / deltaX;
  LENN = LEN+1;
  /**/
  step = 0;

  /* dynamic memeory allocation */
  u = (double *)malloc((LEN + 2) * sizeof(double));
  F = (double *)malloc((LEN + 1) * sizeof(double));
  if( u == NULL || F == NULL ) {
    fprintf(stderr, "Cannot allocate memory\n\n");
    exit(-1);
  }

  u_min = 0.5;
  u_max = 1.0;

  /* initial conditions */
  for (i = 1; i < LENN; i++)
    u[i] = (i > 11 && i <= 21)? u_max: u_min;

  /* time stepping */
  while (step < numstep) {

    if(!(++step%10))
      ;
    /* 
       printf( "%d\n", step );
    */

    /* boundary conditions */
    u[0]     = u_min;
    u[LEN+1] = u_max;

    /* flux */

    /* 1st order in space, 1st order in time */
    /* flux function */
    if (order == LOW) {		
      for (i = 0; i < LENN; i++) {
        F[i] = Fun(u[i]);
      }
    }

    /* quasi-3rd order in space, 2nd order in time */
    /* differences */
    else { // (order = HIGH)
      
      for (i = 0; i < LENN; i++) {
        m = u[i+1] - u[i];
        w = u[i] - u[i-1];
        /* limiting procedure */
        M = minmod( m, w );
        W = minmod( w, m );
        /* derivatives at the left cell's centroid */
        du_dx  = ( M + W ) / ( deltaX + deltaX );
        du_dx2 = ( M - W ) / ( deltaX * deltaX );
        /* estimate of characterisic speed */
        u0 = u[i] + 0.5 * du_dx * deltaX
	    + 0.25 * FI * du_dx2 * deltaX * deltaX;
        /* */
        x = 0.5 * ( deltaX - u0 * deltaT );
        /* reconstructed u at the given point */
        u0 = u[i] + du_dx * x + FI * du_dx2 * x * x;
        /* one more iteration? */
        /*
	x = 0.5 * ( deltaX - u0 * deltaT );
	u0 = u[i] + du_dx * x + FI * du_dx2 * x * x;
        */
        /* flux function */
        F[i] = Fun( u0 );
      } 
      
    } // end else

    /* evolution */
    for (i = 1; i < LENN; i++)
      u[i] += deltaT_X * (F[i-1] - F[i]);

  } /* end while() */

  /*
  printf("%d\n%d\n", LEN+2, 1);
  */
  printf("%1.6f %1.6f \n", 0., u_min - 0.1 * (u_max - u_min));
  for (i = 1; i < LENN; i++)
    printf("%1.6f %1.6f \n", (i - 0.5) * deltaX, u[i]);
  printf("%1.6f %1.6f \n", LEN * deltaX, u_max + 0.1 * (u_max - u_min));

  return 0;

} /* end main() */
  
